import pandas as pd


class DataframeUtils(object):
    @classmethod
    def get_cuc_from_doc_in_df(cls, df, column_doctype, column_doc, column_cuc):
        df[column_cuc] = df[column_doctype].apply(str) + '-' + df[
            column_doc].apply(str)
        return df

    @classmethod
    def print_basic_info(cls, df):
        print(len(df))
        print(len(df.columns))
        print(df.columns)

    @classmethod
    def dynamic_mean_by_n_months(cls, df, column_name, column_id, n):
        group_by_data = df.set_index(str(column_id), append=True).groupby(level=1)

        # resulting_series_with_mean = group_by_data[column_name].apply(
        #     pd.rolling_mean, n, 1).reset_index(str(column_id))
        #return resulting_series_with_mean[column_name]

        resulting_series_with_mean = group_by_data[column_name].apply(
            pd.rolling_mean, n, 1)
        return pd.Series(resulting_series_with_mean.values)

    @classmethod
    def get_categorical_columns(cls, df, columns_to_exclude):
        return \
            [x for x in list(df.select_dtypes(include=['object', 'category']))
             if x not in columns_to_exclude]

    @classmethod
    def get_numerical_columns(cls, df, columns_to_exclude):
        return \
            [x for x in list(df.select_dtypes(include=['integer', 'floating']))
             if x not in columns_to_exclude]

    @classmethod
    def get_columns_by_types(cls, df, columns_to_exclude, ordinal_columns,
                             print_flag=False):
        valid_columns = [x for x in df.columns if x not in columns_to_exclude]
        categorical_columns = \
            cls.get_categorical_columns(df, columns_to_exclude)
        numerical_columns = \
            cls.get_numerical_columns(df, columns_to_exclude)
        other_columns = \
            [x for x in valid_columns if x not in categorical_columns
             and x not in numerical_columns]
        nominal_columns = \
            [x for x in categorical_columns if x not in ordinal_columns]

        if print_flag:
            print('Categóricas:\n', categorical_columns)
            print('\nCategóricas Ordinal:\n', ordinal_columns)
            print('\nCategóricas Nominal:\n', nominal_columns)
            print('\nNuméricas:\n', numerical_columns)
            print('\nOtros:\n', other_columns)
            print('\nExcluidos:\n', columns_to_exclude)

        return valid_columns, categorical_columns, \
               numerical_columns, nominal_columns



import warnings


class Decorators(object):

    __warning_header = 'Decorator Warning: '
    __warning_messages = {
        'not_valid_column': 'Some columns names are not in the related '
                            'dataframe.',
        'invalid_types': 'Not valid types.'
    }

    @classmethod
    def __launch_warning(cls, warning_type):
        warnings.warn(
            cls.__warning_header +
            cls.__warning_messages[warning_type]
        )

    @classmethod
    def __validate_columns_list(cls, df, columns_list):
        for c in columns_list:
            if c not in df.columns.values:
                return False
        return True

    @classmethod
    def __get_param_from_arguments(cls, i, args, kwargs):
        if i < len(args):
            return args[i]
        else:
            kwargs_as_list = list(kwargs.items())
            return kwargs_as_list[i-len(args)][1]

    @classmethod
    def validate_columns(cls, df_arg_pos, column_arg_pos):
        def validate_column_generator(function_to_decorate):
            def inner_function(*args, **kwargs):
                df_to_validate = \
                    cls.__get_param_from_arguments(df_arg_pos, args, kwargs)
                columns_to_validate = \
                    cls.__get_param_from_arguments(column_arg_pos, args, kwargs)

                if isinstance(columns_to_validate, str):
                    if columns_to_validate in df_to_validate.columns.values:
                        return function_to_decorate(*args, **kwargs)
                    else:
                        cls.__launch_warning('not_valid_column')
                        return df_to_validate
                elif isinstance(columns_to_validate, list):
                    if cls.__validate_columns_list(df_to_validate,
                                                   columns_to_validate):
                        return function_to_decorate(*args, **kwargs)
                    else:
                        cls.__launch_warning('not_valid_column')
                        return df_to_validate
                else:
                    cls.__launch_warning('invalid_types')
                    return function_to_decorate(*args, **kwargs)

            return inner_function
        return validate_column_generator



from datetime import datetime


class FormatUtils(object):

    __str_date_format = '%d/%m/%Y'

    __dict_datetimes = {}
    __dict_dateints = {}

    @classmethod
    def get_year(cls, period):
        return period // 100

    @classmethod
    def get_month(cls, period):
        return period % 100

    @classmethod
    def gap_in_months_for_periods(cls, period_1, period_2):
        year_1 = cls.get_year(period_1)
        year_2 = cls.get_year(period_2)
        month_1 = cls.get_month(period_1)
        month_2 = cls.get_month(period_2)

        if year_1 == year_2:
            basic_difference = abs(month_1 - month_2) - 1
            if basic_difference < 0:
                basic_difference = 0
            return basic_difference
        elif year_1 > year_2:
            greater_year_dif = month_1
            smaller_year_dif = 12 - month_2
            basic_difference = greater_year_dif + smaller_year_dif
            additional_months_difference = (year_1 - year_2 - 1) * 12
            return basic_difference + additional_months_difference - 1
        elif year_1 < year_2:
            greater_year_dif = month_2
            smaller_year_dif = 12 - month_1
            basic_difference = greater_year_dif + smaller_year_dif
            additional_months_difference = (year_2 - year_1 - 1) * 12
            return basic_difference + additional_months_difference - 1

    @classmethod
    def get_difference_in_months(cls, datetime1, datetime2):
        difference = datetime1 - datetime2
        difference = abs(difference)
        return difference.days//30

    @classmethod
    def format_date_into_integer(cls, raw_date):
        if raw_date in cls.__dict_dateints:
            return cls.__dict_dateints[raw_date]
        else:
            datetime_obj = datetime.strptime(raw_date, cls.__str_date_format)
            if datetime_obj.month > 9:
                current_month = datetime_obj.month
            else:
                current_month = '0'+str(datetime_obj.month)

            formatted_date = '{year}{month}'.format(year=datetime_obj.year,
                                                    month=current_month)
            integer_date = int(formatted_date)
            cls.__dict_dateints[raw_date] = integer_date
            return integer_date

    @classmethod
    def get_datetime(cls, raw_date):
        if raw_date in cls.__dict_datetimes:
            return cls.__dict_datetimes[raw_date]
        else:
            new_datetime = datetime.strptime(raw_date, cls.__str_date_format)
            cls.__dict_datetimes[raw_date] = new_datetime
            return new_datetime



#from setuptools import setup
from distutils.core import setup


setup(
    name='rimac-analytics',
    version='0.3.1',
    description='Rimac Analytics API',
    author='Diego y Ricardito',
    author_email='ddce.2005@gmail.com',
    zip_safe=False,
    url='https://bitbucket.org/rimac-analytics/rimac-analytics-api',
    license='MIT',
    packages=['utils', 'decorators', 'common'],
    python_requires='>=3'
)
